/** This file is part of Reactor.
 *  Copyright (c) 2020,2021 Kedron Holdings LLC, All Rights Reserved.
 *  Reactor is not public domain or open source. Distribution or derivative works are expressly prohibited.
 */

export const version = 21250;

import api from '/client/ClientAPI.js';
import { mixin } from '/client/util.js';

import Observer from '/client/Observer.js';

import * as Common from './reactor-ui-common.js';
/* Frequently used */
var idSelector = Common.idSelector;

export default class Tab {
    constructor( id, $container ) {
        this.id = id;
        this.$parent = $container;

        $( 'div#' + idSelector( this.id ) + '.re-tab-container' ).remove();
        this.$tab = $( '<div></div>' )
            .hide()
            .attr( 'id', this.id )
            .data( 'tabobject', this )
            .addClass( 're-tab-container' )
            .appendTo( this.$parent );

        this.$tab.on( 'activate', this.activate.bind( this ) );
        this.$tab.on( 'suspend', this.suspend.bind( this ) );

        /* Our styles. */
        if ( 0 === $( 'link#' + idSelector( this.id + '-styles' ) ).length ) {
            let tag = this.id.replace( /^tab-/, "" );
            $('head').append( '<link id="' + this.id + '-styles" href="lib/css/reactor-ui-' + tag + '.css" rel="stylesheet">' );
        }
    }

    /* Observer */
    toString() {
        return this.constructor.name;
    }

    getTabElement() {
        return this.$tab;
    }

    activate( event ) {
    }

    suspend( event ) {
        // console.log( `Tab ${this.id} super.suspend() running` );
        // console.log(event);

        /*
        var $tab = $( event.target );
        if ( this.id !== $tab.attr( 'id' ) || !$tab.hasClass( "re-tab-container" ) ) {
            console.log( "    event is not for the tab, but a child" );
            return;
        }
        */

        if ( ! this.canSuspend() ) {
            return false;
        }

        this.suspending();

        this.$tab.empty();

        this.unsubscribe();
        api.off( '.' + this.id );

        return true;
    }

    canSuspend() {
        return true;    /* default implementation */
    }

    /** Called when a suspend is in progress */
    suspending() {
        /* Default implementation does nothing. */
    }
}

mixin( Tab, Observer );
